<?php
   /*
   Plugin Name: Woocommerce Subscriptions Renewal Reminder
   Plugin URI: http://qwebmaster.com/woocommerce-subscriptions-renewal-reminder/
   Description: Reminds subscriptions customers X days before the subscription renewal date
   Version: 1.2
   Requires at least: 4.0
   Tested up to: 5.5.1
   Requires PHP: 7.3
   Author: Qwebmaster
   Author URI: http://qwebmaster.com
   */

   if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

    // Load the settings class
    require_once( 'wsrr_settings.php' );
    // Load action scheduler v3.1.6
    require_once( plugin_dir_path( __FILE__ ) . '/libraries/action-scheduler/action-scheduler.php' );

   /**
    *  Add a custom email to the list of emails WooCommerce should load
    *
    * @since 0.1
    * @param array $email_classes available email classes
    * @return array filtered available email classes
    */




 /**
  *  When a subscription order status is changed, reove the email from queue
  *
  * @since 0.1
  */
function wsrr_subscription_status_updated($subscription){

  // Renewal Date
  $trigger_date = $subscription->get_time( 'next_payment' );

  $end_date = $subscription->get_date( 'end', 'site' );
  if (empty($trigger_date) && !empty($end_date)) {
    $email_body = $options['email_body_cheque'];
    $trigger_date = $subscription->get_date( 'end', 'site' );
    //wsrr_email_date_debug('End date trigger');
  }

  $hook = 'wsrr_renewal_reminder_email';
  $args = array(
    'subscription_id' => $subscription->get_id(),
    'payment_method' => $subscription->payment_method,
    //'renewal_date'    => gmdate("Y-m-d\ H:i:s", $trigger_date)
  );
  $group = 'wsrr';
  //wsrr_email_date_debug($args);
  wsrr_unschedule_scheduled_actions($hook,$args,$group);
}
// When status is set to cancelled
add_action( 'woocommerce_subscription_status_cancelled', 'wsrr_subscription_status_updated',10,1);
// In case subscription expires
add_action( 'woocommerce_subscription_status_expired', 'wsrr_subscription_status_updated',10,1);
// If subscription is set to on hold
add_action( 'woocommerce_subscription_status_on-hold', 'wsrr_subscription_status_updated',10,1);



function saveYouTubeInfo($meta_id, $post_id, $meta_key='',$meta_value=''){

    // Stop if not the correct meta key
    if ( $meta_key == '_schedule_next_payment' || $meta_key == '_schedule_end') {
        //wsrr_email_date_debug('META KEY:'.$meta_key.' POST ID:'.$post_id);
        wsrr_woo_get_all_active_subsctiptions();
    }

    // Function code goes here.
}

add_action('updated_post_meta', 'saveYouTubeInfo', 10, 4);





/**
 *  When an order is updated (manually) remove the scheduled action
 *
 * @since 0.5
 */
add_action( 'woocommerce_process_shop_order_meta', 'wsrr_woocommerce_process_shop_order', 10, 2 );
function wsrr_woocommerce_process_shop_order ( $post_id, $post ) {
      if ( ! is_admin()) {
        return;
      }
      //wsrr_email_date_debug($post->post_type);

      if ( $post->post_type == 'shop_subscription') {
        $subscription = new WC_Subscription( $post_id );
        //wsrr_email_date_debug($subscription);


        // Renewal Date
      //  $trigger_date = $subscription->get_time( 'next_payment' );

        //$end_date = $subscription->get_date( 'end', 'site' );
        //if (empty($trigger_date) && !empty($end_date)) {
          //$email_body = $options['email_body_cheque'];
          //$trigger_date = $subscription->get_date( 'end', 'site' );
          //wsrr_email_date_debug('End date trigger');
        //}

        $hook = 'wsrr_renewal_reminder_email';
        $args = array(
          'subscription_id' => $subscription->get_id(),
          'payment_method' => $subscription->payment_method,
        //  'renewal_date'    => gmdate("Y-m-d\ H:i:s", $trigger_date)
        );
        $group = 'wsrr';

        wsrr_unschedule_scheduled_actions($hook,$args,$group);

        //wsrr_woo_get_all_active_subsctiptions();
      }


}



/**
 *  When a subscription order status is set to active run the main hook "wsrr_renewal_reminder_scheduler"
 *
 * @since 0.1
 */
function wsrr_subscription_status_active(){
  wsrr_woo_get_all_active_subsctiptions();
}
add_action( 'woocommerce_subscription_status_active', 'wsrr_subscription_status_active' );


/**
 *  Schedule main wsrr_renewal_reminder_scheduler action
 *
 * @since 0.1
 */
add_action( 'init', 'wsrr_renewal_reminder_scheduler' );
function wsrr_renewal_reminder_scheduler(){
  $options = get_option( 'wsrr_option' );
  // skip email and adding post meta if Debug is checked
  if ($options['wsrr_enable'] != 1) {
    // If not enabled
    return;
  }
  // Check if the action is alredy scheduled
  $interval_in_seconds = 24*60*60; // 24h in seconds
  $hook = 'wsrr_renewal_reminder_scheduler';
  $args = array(
    //'subscription_id' => $subscription_id,
  );

  $group = '';

  $params = array();
  if ( is_array($args) ) {
    $params['args'] = $args;
  }
  if ( !empty($group) ) {
    $params['group'] = $group;
  }

  $action = ActionScheduler::store()->find_action( $hook, $params,$group );

  if (wsrr_check_scheduled_actions($hook,$args,$group)){
     // Action alredy scheduled do nothing
     // wsrr_unschedule_scheduled_actions($hook,$args,$group);
    return;
  } else {
   // Schedule Action
   as_schedule_recurring_action( time(), $interval_in_seconds, $hook, $args, $group );
  }
}


/**
 *  Triggers once every X hours, scans all subscriptions
 *
 * @since 0.1
 * @essential
 */
add_action( 'wsrr_renewal_reminder_scheduler', 'wsrr_renewal_reminder', 10, 1  );
function wsrr_renewal_reminder(){
  wsrr_woo_get_all_active_subsctiptions();
}


/**
*  Trigger when an email is sent out via the Actions Scheduler
*
* @since 0.1
* @param $order_id
*/
function wsrr_renewal_reminder_email( $subscription_id ) {

 $subscription = new WC_Subscription( $subscription_id );
 $options = get_option( 'wsrr_option' );

// Template
//if ($subscription->payment_method == 'stripe') {
  // $email_template = 'wsff-renewal-reminder-stripe.php';
//} elseif ($subscription->payment_method == 'cheque') {
  // $email_template = 'wsff-renewal-reminder-cheque.php';
//} else {
   //$email_template = 'wsff-renewal-reminder.php';
//}

/*
// Additional Logic
// If only exparation date is set, send the checque email
$next_payment = $subscription->get_time( 'next_payment' );
$end_date = $subscription->get_date( 'end', 'site' );
if (empty($next_payment) && !empty($end_date)) {
  $email_template = 'wsff-renewal-reminder-cheque.php';
//  wsrr_email_date_debug('End date trigger');
}
*/

$email_template = 'wsff-renewal-reminder.php';

 // Load WC mailer
 $mailer = WC()->mailer();
 // Customer email
 $email = $subscription->get_billing_email();
 $recipient = $email;
 $subject = __($options['email_subject'], 'wsrr');
 $content = wsrr_renewal_reminder_email_template( $subscription, $subject, $mailer, $email_template );
 $headers = array(
  'Content-Type: text/html\r\n',
//  'From: alex646515@gmail.com',
  'CC: ' . $options['email_cc'],
  'BCC: ' .  $options['email_bcc']
 );

 // If wsrr_debug_subscription_id is set will only work for that subscription ID
 if (!empty($options['wsrr_debug_subscription_id']) && $options['wsrr_debug_subscription_id'] != 0 ) {

     if ($subscription_id == $options['wsrr_debug_subscription_id']) {
         //wsrr_email_date_debug('ONLY FIRE FOR ORDER ID:'.$options['wsrr_debug_subscription_id']);
     } else {
         return;
     }

 }

 // skip email and adding post meta if Debug is checked
 if ($options['wsrr_debug'] != 1) {

   $mailer->send( $recipient, $subject, $content, $headers );
   // Set the payment count to mark this email as sent
   $completed_paymet_count = $subscription->get_completed_payment_count();
   add_post_meta( $subscription_id, 'subscription_renewal_reminder', $completed_paymet_count, true );
 } else {
   //wsrr_email_date_debug($email);
 }

}
add_action( 'wsrr_renewal_reminder_email', 'wsrr_renewal_reminder_email', 10, 1 );

/**
*  Override the default woocommerce email template
*
* @since 0.1
* @param $order
* @param $heading = false
* @param $mailer
*/
function wsrr_renewal_reminder_email_template( $order, $heading = false, $mailer, $email_template) {
$options = get_option( 'wsrr_option' );
//$template = 'emails/admin-new-order.php';
$template = 'emails/'.$email_template;
return wc_get_template_html( $template, array(
  'order'         => $order,
  'email_heading' => $options['email_title'],
  'sent_to_admin' => true,
  'plain_text'    => false,
  'email'         => $mailer
),
'',
wsrr_plugin_path() . "/templates/" );
}


/**
 *  Get plugin path
 *
 * @since 0.1
 * @essential
 */
function wsrr_plugin_path(){
    return untrailingslashit( plugin_dir_path( __FILE__ ) );
}


/**
*  Get all WooCommerce Subscriptions
*
* @since 0.1
*/
function wsrr_woo_get_all_active_subsctiptions(){
global $wpdb;
$posts = $wpdb->prefix . "posts";
// Get all subscriptions ID's
$subscriptions_ids = $wpdb->get_col("
   SELECT ID
   FROM $posts
   WHERE post_type LIKE 'shop_subscription'
");

 foreach($subscriptions_ids as $subscription_id){

     // Get an instance of the WC_Subscription object
     $subscription = new WC_Subscription( $subscription_id );
       // Only Active subscriptions
        //wsrr_email_date_debug('foreach($subscriptions_ids as $subscription_id){'.$subscription_id);
       if ( $subscription->get_status() == 'active' ) {

       $options = get_option( 'wsrr_option' );

       $billing_period = $subscription->billing_period;

       // Exclude Yearly Subscriptions
      if (!empty($options['exclude_year']) && $options['exclude_year'] != 0 ) {
        if ($billing_period == 'year') {
          continue;
        }
      }

        // Exclude Monthley Subscriptions
       if (!empty($options['exclude_month']) && $options['exclude_month'] != 0 ) {
         if ($billing_period == 'month') {
           continue;
         }
       }

       // Exclude Weekley Subscriptions
      if (!empty($options['exclude_week']) && $options['exclude_week'] != 0 ) {
        if ($billing_period == 'week') {
          continue;
        }
      }


         //var_dump($subscription);
         $trigger_date = $subscription->get_time( 'next_payment' );
         //wsrr_email_date_debug('SUBSCRIPTION IS ACTIVE: $trigger_date = $subscription->get_time(next_payment);'.$subscription_id);
         if (!$trigger_date ) {
           //wsrr_email_date_debug('if (!$trigger_date ) {'.$subscription_id);
           // If date returns 0 try to get the expiration date
           $trigger_date = $subscription->get_date( 'end', 'site' );

         }

         if ( !is_numeric($trigger_date) && $trigger_date != '0') {
             $trigger_date = strtotime($trigger_date);
             // wsrr_email_date_debug($trigger_date);
         }

         // Check if this is a valid timestamp
         // Some orders return the date and not a UNIX timestamp
         // Make sure it's not a 0
         if ( is_numeric($trigger_date) && (int)$trigger_date == $trigger_date  && $trigger_date != '0') {
          // var_dump('VALID');
          //wsrr_email_date_debug('if ( is_numeric($trigger_date) && (int)$trigger_date == $trigger_date  && $trigger_date != 0) {'.$subscription_id);
          //$options = get_option( 'wsrr_option' );

          // If wsrr_debug_subscription_id is set will only work for that subscription ID
          if (!empty($options['wsrr_debug_subscription_id']) && $options['wsrr_debug_subscription_id'] != 0 ) {

              if ($subscription_id == $options['wsrr_debug_subscription_id']) {
                  //wsrr_email_date_debug('ONLY FIRE FOR ORDER ID:'.$options['wsrr_debug_subscription_id']);
              } else {
                  continue;
              }


          }


           // #1
           // Check if renewal date is within X days of today
           // Offset the renewal ofset another 10 days so actions can be scheduled ahead of time
           $renewal_offset = $options['renewal_days'];
           //$renewal_offset = 1 + 10;
           $trigger = 0;
           // Skip reminders that should have been sent out based on X days.
           if ($options['skip_overdue'] == 1) {
             if( $trigger_date < strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day') && $trigger_date >= strtotime('+'.$renewal_offset.' day') ) {
                $trigger = 1;
                 //wsrr_email_date_debug('renewal_days_offset 1');
               }
             //wsrr_email_date_debug('ID:'.$subscription_id.'TR DATE:'.$trigger_date.'$renewal_offset'.$renewal_offset.'$options[renewal_days_offset]'.$options['renewal_days_offset'].'STR Time'.strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day').'STR Time 2'.strtotime('+'.$renewal_offset.' day'));
              // wsrr_email_date_debug($subscription->get_time( 'next_payment' ));
           } else {
             if( $trigger_date < strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day') ) {
                $trigger = 1;
                //wsrr_email_date_debug('renewal_days_offset 2');
             }
           }

           if( $trigger == 1) {
          // Only orders X Days or MORE
          //if( $trigger_date < strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day') && $trigger_date >= strtotime('+'.$renewal_offset.' day') ) {
            //wsrr_email_date_debug($trigger_date.'TR DATE');
            //wsrr_email_date_debug(strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day'));
          // All orders X days or LESS and MORE
        //  if( $trigger_date < strtotime('+'.$renewal_offset + $options['renewal_days_offset'] .' day') ) {
           //wsrr_email_date_debug('if( $trigger_date < strtotime('+'.$renewal_offset + $options[renewal_days_offset] . day) ) {'.$subscription_id);
            $subscription_id = $subscription->get_id();
            $completed_paymet_count = $subscription->get_completed_payment_count();
            $status = get_post_meta( $subscription_id, 'subscription_renewal_reminder', true );

            //Check if this renewal email has alredy been sent
            if ( ! empty( $status ) ) {
              //wsrr_email_date_debug('if ( ! empty( $status ) ) {'.$subscription_id);
             // If an email has alredy been sent for this subscription
             // count the number of completed orders and make sure they don't match the current completed orders
              if ($completed_paymet_count <= $status) {
                  //wsrr_email_date_debug('  if ($completed_paymet_count <= $status) {'.$subscription_id);
                // Email alredy sent skip
                //var_dump('Email alredy sent (uncomment break; )');
                continue;
              }
            }

            //if ()
            //var_dump($status);

            // If the scheduled action is less than X days when this plugin is installed
            // Run the action right away
            $timestamp = strtotime('-'.$renewal_offset.' day', $trigger_date);

            $interval_in_seconds = 24*60*60; // 24h in seconds
            $hook = 'wsrr_renewal_reminder_email';
            $args = array(
              'subscription_id' => $subscription_id,
              'payment_method' => $subscription->payment_method,
              //'renewal_date'    => gmdate("Y-m-d\ H:i:s", $trigger_date)
            );
            $group = 'wsrr';


            // #2
            // Check if email is alredy in scheduled actions
            // var_dump(wsrr_check_scheduled_actions($hook,$args).'$$$$$$$');
            // Check if there is an action scheduled for this email
            if (wsrr_check_scheduled_actions($hook,$args,$group)){
              //var_dump('Action Scheduled');
               //wsrr_unschedule_scheduled_actions($hook,$args,$group);
              // Email action alredy set
                //wsrr_email_date_debug('if (wsrr_check_scheduled_actions($hook,$args,$group)){'.$subscription_id);
              continue;
            }

            // #3
            // Proceed to scheduling the renewal email
            // as_schedule_recurring_action( $timestamp, $interval_in_seconds, $hook, $args, $group );
            // wc_unschedule_action( 'sfn_followup_emails', $param, 'fue' );
            // wc_schedule_single_action( $send_on, 'sfn_followup_emails', $param, 'fue' );
             wc_schedule_single_action($timestamp, $hook, $args, $group);
             // Set the payment count  when scheduling the action
            // add_post_meta( $subscription_id, 'subscription_renewal_reminder', $completed_paymet_count, true );

                //var_dump('LESS THAN '.$renewal_offset.' DAYS');
                //echo gmdate("Y-m-d\TH:i:s\Z", strtotime('+9 day'));
          }

         } else {
           //var_dump('IN VALID!');
         }

           //echo gmdate("Y-m-d\TH:i:s\Z", $trigger_date);
           //var_dump($subscription_id.' TIME '.$trigger_date);


       }
 }
}


/*
   $args = array(
       'post_type'=> 'scheduled-action',
       'order'    => 'ASC'
       );

   $the_query = new WP_Query( $args );
   if($the_query->have_posts() ) : while ( $the_query->have_posts() ) : $the_query->the_post();
*/


 /**
  *  Check Scheduled Actions
  *
  * @since 0.1
  */
function wsrr_check_scheduled_actions($hook,$args,$group) {
  //$hook = 'test_action';
  $params = array();
  if ( is_array($args) ) {
    $params['args'] = $args;
  }
  if ( !empty($group) ) {
    $params['group'] = $group;
  }

  $action = ActionScheduler::store()->find_action( $hook, $params,$group );
  return $action;
}

 /**
  *  Remove Scheduled Actions
  *
  * @since 0.1
  */
function wsrr_unschedule_scheduled_actions($hook,$args,$group) {
  //$hook = 'test_action';
  $params = array();
  if ( is_array($args) ) {
    $params['args'] = $args;
  }
  if ( !empty($group) ) {
    $params['group'] = $group;
  }

  $action = wc_unschedule_action( $hook, $param, $group );
  return $action;
}






add_action( 'wp_head', 'wsrr_head' );
function wsrr_head(){
  $options = get_option( 'wsrr_option' );
  //$options['wsrr_enable']
  //$options['renewal_days']
  //$options['renewal_days_offset']
  //$options['email_bcc']
  //$options['email_cc']



/*
array(5) {
  ["renewal_days"]=>
  int(10)
  ["renewal_days_offset"]=>
  int(10)
  ["email_bcc"]=>
  string(19) "alex@qwebmaster.com"
  ["email_cc"]=>
  string(20) "akex646515@yahoo.com"
  ["wsrr_enable"]=>
  string(1) "1"
}
*/

}

//
// dEBUG USE  ONLY
//
    /**
     *  Human Readable array
     *
     * @since 0.1
     * @debug
     */
   function wsrr_print_array($aArray) {
     echo '<pre>';
     print_r($aArray);
     echo '</pre>';
   }



    /**
     *  Send data via email
     *
     * @since 0.1
     * @debug
     */
   function wsrr_email_date_debug($data) {
    ob_start();
    var_dump($data);
    $result = ob_get_clean();
    mail('alex646515@gmail.com', 'Debug data',$result);
     //mail('alex@qwebmaster.com', 'Debug data',$result);
    //wp_mail('alex@qwebmaster.com', 'Debug data',$result);
   }
